#include "ImageManager.h"

ImageManager::ImageManager() : AssetManager<Image> ("ImageManager.log")
{
    ilInit();
    iluInit();
}

ImageManager::~ImageManager()
{
    
}

Image* ImageManager::Load(char *filename, unsigned int &userID, void *userData)
{
    unsigned int ilID, glID;
    ilGenImages(1, &ilID);
    glGenTextures(1, &glID);
    ilBindImage(ilID);
    glBindTexture(GL_TEXTURE_2D, glID);
    if(ilLoad(IL_TYPE_UNKNOWN, filename)==IL_FALSE)
    {
        printf("EE - Could not load image '%s'\n", filename);
        glDeleteTextures(1, &glID);
        ilDeleteImages(1, &ilID);
        return NULL;
    }
    if(ilConvertImage(IL_RGB, IL_UNSIGNED_BYTE)==IL_FALSE)
    {
        printf("EE - Could not convert image '%s' in RGB format.\n", filename);
        glDeleteTextures(1, &glID);
        ilDeleteImages(1, &ilID);	
	return NULL;
    }
    unsigned char depth = ((ImageLoadRequest*)userData)->depth;
    int width = ilGetInteger(IL_IMAGE_WIDTH);
    int height = ilGetInteger(IL_IMAGE_HEIGHT);
    int newWidth = (int)((double)width/(pow(2, depth)));
    int newHeight = (int)((double)height/(pow(2, depth)));
    iluImageParameter(ILU_FILTER, ILU_NEAREST);  //<--- this is the default
    if(iluScale(newWidth, newHeight, 3)==IL_FALSE)
    {
        printf("EE - Could not scale image '%s' Could have impletemented error string readback, but meh...\n", filename);
        glDeleteTextures(1, &glID);
        ilDeleteImages(1, &ilID);
        return NULL;
    }
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, newWidth, newHeight, 0, GL_RGB, GL_UNSIGNED_BYTE, ilGetData());
//    gluBuild2DMipmaps(GL_TEXTURE_2D, GL_RGB, newWidth, newHeight, GL_RGB, GL_UNSIGNED_BYTE, ilGetData());
    Image *created = new Image(glID, depth, newWidth, newHeight);
    ilDeleteImages(1, &ilID);
    return created;
}

